package de.fraunhofer.iem.dataprovider.taskManager

import de.fraunhofer.iem.dataprovider.logger.getLogger
import jakarta.annotation.PostConstruct
import jakarta.annotation.PreDestroy
import kotlinx.coroutines.*
import kotlinx.coroutines.channels.Channel
import org.springframework.stereotype.Component
import kotlin.coroutines.CoroutineContext

enum class TaskType{
    REPO_CHANGED
}
data class Task(val type: TaskType, val payload: Any)
@Component
class TaskManager : CoroutineScope {

    private val job = SupervisorJob()
    override val coroutineContext: CoroutineContext
        get() = Dispatchers.Default + job

    // TODO: this could be dangerous to give it unlimited memory
    private val tasks = Channel<String>(Channel.UNLIMITED)
    private val numCores = Runtime.getRuntime().availableProcessors()
    private val logger = getLogger(javaClass)

    @PostConstruct
    fun startWorkers() {
        logger.info("Starting $numCores workers.")
        repeat(numCores-1) {
            logger.info("[${Thread.currentThread().name}] launching coroutine")
            launchWorker(it)
        }
    }

    private fun launchWorker(id: Int) = launch {
        for (task in tasks) {
            launch {
                logger.debug("[${Thread.currentThread().name}] Processor #$id received $task")
                executeTask(task)
                logger.debug("[${Thread.currentThread().name}] Processor #$id finished $task")
            }
        }
    }

    private suspend fun executeTask(task: String) {
        // simulate a long running task
        delay(5000)
    }

    fun addTask(task: String) {
        launch {
            tasks.send(task)
        }
    }

    @PreDestroy
    fun stop() {
        job.cancel()
    }

}
